@@ -1,6 +1,6 @@
 /* vi:set ts=4: */
 /*
- * Copyright (C) 2021-2024 Tetsuya Isaki
+ * Copyright (C) 2021-2025 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -144,4 +144,10 @@ typedef int64_t		int64;
 #define SLOW_ARCH
 #endif
 
+#if defined(SLOW_ARCH)
+#define IMAGE_BUFSIZE	(4096)
+#else
+#define IMAGE_BUFSIZE	(16384)
+#endif
+
 #endif // !sayaka_header_h
