@@ -1,6 +1,6 @@
 /* vi:set ts=4: */
 /*
- * Copyright (C) 2021-2025 Tetsuya Isaki
+ * Copyright (C) 2021-2026 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -392,17 +392,28 @@ image_get_preferred_size(
 }
 
 // ローダごとにサポートしているファイル形式をビットマップフラグにしたもの。
+#define LOADERMAP_bmp	(1U << IMAGE_LOADER_BMP)
 #define LOADERMAP_gif	(1U << IMAGE_LOADER_GIF)
+#define LOADERMAP_ico	(1U << IMAGE_LOADER_ICO)
 #define LOADERMAP_jpeg	(1U << IMAGE_LOADER_JPEG)
 #define LOADERMAP_jxl	(1U << IMAGE_LOADER_JXL)
+#define LOADERMAP_mag	(1U << IMAGE_LOADER_MAG)
 #define LOADERMAP_png	(1U << IMAGE_LOADER_PNG)
+#define LOADERMAP_pnm1	(1U << IMAGE_LOADER_PNM1)
+#define LOADERMAP_pnm2	(1U << IMAGE_LOADER_PNM2)
+#define LOADERMAP_pnm3	(1U << IMAGE_LOADER_PNM3)
+#define LOADERMAP_pnm4	(1U << IMAGE_LOADER_PNM4)
+#define LOADERMAP_pnm5	(1U << IMAGE_LOADER_PNM5)
+#define LOADERMAP_pnm6	(1U << IMAGE_LOADER_PNM6)
 #define LOADERMAP_tiff	(1U << IMAGE_LOADER_TIFF)
 #define LOADERMAP_webp	(1U << IMAGE_LOADER_WEBP)
+#define LOADERMAP_ypic	(1U << IMAGE_LOADER_YPIC)
 #define LOADERMAP_stb	((1U << IMAGE_LOADER_BMP)	| \
 						 (1U << IMAGE_LOADER_GIF)	| \
 						 (1U << IMAGE_LOADER_JPEG)	| \
 						 (1U << IMAGE_LOADER_PNG)	| \
-						 (1U << IMAGE_LOADER_PNM))
+						 (1U << IMAGE_LOADER_PNM5)	| \
+						 (1U << IMAGE_LOADER_PNM6))
 
 // サポートしているローダ。処理順に並べること。
 static const struct {
@@ -430,9 +441,30 @@ static const struct {
 #if defined(USE_GIFLIB)
 	ENTRY(gif, giflib, LOADERMAP_gif),
 #endif
+#if defined(USE_BUILTIN_BMP) && defined(SIXELV)
+	ENTRY(bmp, builtin, LOADERMAP_bmp),
+#endif
 #if defined(USE_LIBTIFF) && defined(SIXELV)
 	ENTRY(tiff, libtiff, LOADERMAP_tiff),
 #endif
+#if defined(USE_BUILTIN_PNM) && defined(SIXELV)
+	ENTRY(pnm5, builtin, LOADERMAP_pnm5),
+	ENTRY(pnm6, builtin, LOADERMAP_pnm6),
+	// ASCII と P4 は実用上の価値はないので優先度後ろでいい。
+	ENTRY(pnm1, builtin, LOADERMAP_pnm1),
+	ENTRY(pnm2, builtin, LOADERMAP_pnm2),
+	ENTRY(pnm3, builtin, LOADERMAP_pnm3),
+	ENTRY(pnm4, builtin, LOADERMAP_pnm4),
+#endif
+#if defined(USE_BUILTIN_ICO) && defined(SIXELV)
+	ENTRY(ico,  builtin, LOADERMAP_ico),
+#endif
+#if defined(USE_BUILTIN_MAG) && defined(SIXELV)
+	ENTRY(mag,  builtin, LOADERMAP_mag),
+#endif
+#if defined(USE_BUILTIN_YPIC) && defined(SIXELV)
+	ENTRY(ypic, builtin, LOADERMAP_ypic),
+#endif
 #if defined(USE_STB_IMAGE)
 	ENTRY(stb, stb_image, LOADERMAP_stb),
 #endif
@@ -554,21 +586,18 @@ image_convert_to16(struct image *img)
 
 	if (img->format == IMAGE_FMT_RGB24) {
 		for (uint i = 0; i < count; i++) {
-			uint r, g, b, v;
-			r = (*s8++) >> 3;
-			g = (*s8++) >> 3;
-			b = (*s8++) >> 3;
-			v = (r << 10) | (g << 5) | b;
-			*d16++ = v;
+			uint8 r = *s8++;
+			uint8 g = *s8++;
+			uint8 b = *s8++;
+			*d16++ = RGB888_to_ARGB16(r, g, b);
 		}
 	} else if (img->format == IMAGE_FMT_ARGB32) {
 		for (uint i = 0; i < count; i++) {
-			uint r, g, b, a, v;
-			r = (*s8++) >> 3;
-			g = (*s8++) >> 3;
-			b = (*s8++) >> 3;
-			a = (*s8++);
-			v = (r << 10) | (g << 5) | b;
+			uint8 r = *s8++;
+			uint8 g = *s8++;
+			uint8 b = *s8++;
+			uint8 a = *s8++;
+			uint16 v = RGB888_to_ARGB16(r, g, b);
 			// A(不透明度)が半分以下なら透明(0x8000)とする。
 			if (a < 0x80) {
 				v |= 0x8000;
@@ -578,6 +607,8 @@ image_convert_to16(struct image *img)
 
 		// 実際にあるかどうかは調べてない。
 		img->has_alpha = true;
+	} else if (img->format != IMAGE_FMT_ARGB16) {
+		assert(img->format == IMAGE_FMT_ARGB16);
 	}
 
 	img->format = IMAGE_FMT_ARGB16;
@@ -979,10 +1010,7 @@ image_reduct_highquality_adaptive(image_reductor_handle *ir)
 			RESIZE_STEP(sx0, sx1, rx, xstep);
 
 			ColorRGB c8 = pixel_mean(ir, sy0, sy1, sx0, sx1);
-			uint16 v;
-			v  = (c8.r >> 3) << 10;
-			v |= (c8.g >> 3) <<  5;
-			v |= (c8.b >> 3);
+			uint16 v = RGB888_to_ARGB16(c8.r, c8.g, c8.b);
 			if (__predict_false(c8.a)) {
 				v |= 0x8000;
 			}
@@ -1801,7 +1829,7 @@ image_calc_adaptive_palette(image_reductor_handle *ir, struct image *srcimg,
 			if (__predict_false(r5 > 31)) r5 = 31;
 			if (__predict_false(g5 > 31)) g5 = 31;
 			if (__predict_false(b5 > 31)) b5 = 31;
-			n = (r5 << 10) | (g5 << 5) | b5;
+			n = RGB555_to_ARGB16(r5, g5, b5);
 		}
 		// ピクセル数の計数は 0xffff で頭打ちにしておく。
 		// 65536 ピクセル以上の色が 256 色以上ある時には困るかも知れないが。
