@@ -1,6 +1,6 @@
 /* vi:set ts=4: */
 /*
- * Copyright (C) 2024-2025 Tetsuya Isaki
+ * Copyright (C) 2024-2026 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -180,12 +180,20 @@ enum {
 	IMAGE_LOADER_BLURHASH = 0,
 	IMAGE_LOADER_BMP,
 	IMAGE_LOADER_GIF,
+	IMAGE_LOADER_ICO,
 	IMAGE_LOADER_JPEG,
 	IMAGE_LOADER_JXL,
+	IMAGE_LOADER_MAG,
 	IMAGE_LOADER_PNG,
-	IMAGE_LOADER_PNM,
+	IMAGE_LOADER_PNM1,	// P1(PBM,ASCII)
+	IMAGE_LOADER_PNM2,	// P2(PGM,ASCII)
+	IMAGE_LOADER_PNM3,	// P3(PPM,ASCII)
+	IMAGE_LOADER_PNM4,	// P4(PBM,Bin)
+	IMAGE_LOADER_PNM5,	// P5(PGM,Bin)
+	IMAGE_LOADER_PNM6,	// P6(PPM,Bin)
 	IMAGE_LOADER_TIFF,
 	IMAGE_LOADER_WEBP,
+	IMAGE_LOADER_YPIC,
 	IMAGE_LOADER_MAX,
 };
 // IMAGE_LOADER_* に対応する名前。
@@ -193,12 +201,20 @@ enum {
 	"blurhash",	\
 	"bmp",	\
 	"gif",	\
+	"ico",	\
 	"jpeg",	\
 	"jxl",	\
+	"mag",	\
 	"png",	\
-	"pnm",	\
+	"pbm(P1)",	\
+	"pgm(P2)",	\
+	"ppm(P3)",	\
+	"pbm(P4)",	\
+	"pgm(P5)",	\
+	"ppm(P6)",	\
 	"tiff",	\
-	"webp"
+	"webp",	\
+	"ypic"
 
 struct image
 {
@@ -230,14 +246,20 @@ struct image
 
 // image_read() に対するヒント。
 typedef struct image_read_hint_ {
+	// 必要な読み込みサイズ。
+	// 通常のラスター形式の画像では意味を持たない。
+	// プログレッシブ画像 (で no_progressive == true でない) なら、
+	// このサイズを満たしたところで早期終了してもよい。
+	// Blurhash はこの hint を使わない。
 	ResizeAxis axis;
 	uint width;
 	uint height;
 
+	// プログレッシブ画像であっても最後までデコードする場合は true。
+	bool no_progressive;
+
 	// 複数枚ある場合のページ(フレーム)番号。0 から始まる。
 	uint page;
-
-	bool progressive;
 } image_read_hint;
 
 struct image_opt {
