@@ -1,6 +1,6 @@
 /* vi:set ts=4: */
 /*
- * Copyright (C) 2024-2025 Tetsuya Isaki
+ * Copyright (C) 2024-2026 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,7 +36,20 @@
 
 // テストなどで一時的に無効にしたい場合のため。
 #define USE_BLURHASH
-#define USE_STB_IMAGE
+#define USE_BUILTIN_BMP
+
+#if defined(HAVE_BUILTIN_ICO)
+#define USE_BUILTIN_ICO
+#endif
+#if defined(HAVE_BUILTIN_MAG)
+#define USE_BUILTIN_MAG
+#endif
+#if defined(HAVE_BUILTIN_PNM)
+#define USE_BUILTIN_PNM
+#endif
+#if defined(HAVE_BUILTIN_YPIC)
+#define USE_BUILTIN_YPIC
+#endif
 #if defined(HAVE_GIFLIB)
 #define USE_GIFLIB
 #endif
@@ -49,6 +62,9 @@
 #if defined(HAVE_LIBPNG)
 #define USE_LIBPNG
 #endif
+#if defined(WITH_STB_IMAGE)
+#define USE_STB_IMAGE
+#endif
 #if defined(HAVE_LIBTIFF)
 #define USE_LIBTIFF
 #endif
@@ -68,14 +84,38 @@ typedef struct image *(*image_read_t)(FILE *, const image_read_hint *,
 	extern struct image *image_##name##_read(FILE *,	\
 		const image_read_hint *, const struct diag *)
 
+IMAGE_HANDLER(bmp);
 IMAGE_HANDLER(gif);
+IMAGE_HANDLER(ico);
 IMAGE_HANDLER(jpeg);
 IMAGE_HANDLER(jxl);
+IMAGE_HANDLER(mag);
 IMAGE_HANDLER(png);
+IMAGE_HANDLER(pnm1);
+IMAGE_HANDLER(pnm2);
+IMAGE_HANDLER(pnm3);
+IMAGE_HANDLER(pnm4);
+IMAGE_HANDLER(pnm5);
+IMAGE_HANDLER(pnm6);
 IMAGE_HANDLER(stb);
 IMAGE_HANDLER(tiff);
 IMAGE_HANDLER(webp);
+IMAGE_HANDLER(ypic);
 
 #undef IMAGE_HANDLER
 
+// R5,G5,B5 を内部形式に変換。
+static inline uint16
+RGB555_to_ARGB16(uint8 r, uint8 g, uint8 b)
+{
+	return (r << 10) | (g << 5) | b;
+}
+
+// R8,G8,B8 を内部形式に変換。
+static inline uint16
+RGB888_to_ARGB16(uint8 r, uint8 g, uint8 b)
+{
+	return RGB555_to_ARGB16(r >> 3, g >> 3, b >> 3);
+}
+
 #endif // !sayaka_image_priv_h
