@@ -1,6 +1,6 @@
 /* vi:set ts=4: */
 /*
- * Copyright (C) 2014-2025 Tetsuya Isaki
+ * Copyright (C) 2014-2026 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,8 +41,8 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 
-#define SAYAKA_VERSION "3.8.5"
-#define SAYAKA_RELDATE "2025/12/30"
+#define SAYAKA_VERSION "3.8.6"
+#define SAYAKA_RELDATE "2026/01/28"
 
 enum {
 	DEFAULT_FONT_WIDTH	= 7,
@@ -59,6 +59,7 @@ enum {
 extern struct image_opt imageopt;
 
 static void version(void);
+static void list_supported_images(void);
 static void usage(void);
 static void help_all(void);
 static bool init(void);
@@ -67,7 +68,7 @@ static void progress(const char *);
 static void init_screen(void);
 static void init_ngword(void);
 static void invalidate_cache(void);
-static const char *get_token(const char *);
+static string *get_token(const char *);
 static void signal_handler(int);
 static void sigwinch(bool);
 
@@ -126,6 +127,7 @@ enum {
 	OPT_ipv6,
 	OPT_jis,
 	OPT_light,
+	OPT_list_supported_images,
 	OPT_mathalpha,
 	OPT_max_image_cols,
 	OPT_misskey,
@@ -161,6 +163,7 @@ static const struct option longopts[] = {
 	{ "ipv6",			no_argument,		NULL,	OPT_ipv6 },
 	{ "jis",			no_argument,		NULL,	OPT_jis },
 	{ "light",			no_argument,		NULL,	OPT_light },
+	{ "list-supported-images", no_argument,	NULL,	OPT_list_supported_images },
 	{ "local",			no_argument,		NULL,	'l' },
 	{ "mathalpha",		no_argument,		NULL,	OPT_mathalpha },
 	{ "max-image-cols",	required_argument,	NULL,	OPT_max_image_cols },
@@ -375,6 +378,10 @@ main(int ac, char *av[])
 			opt_bgtheme = BG_LIGHT;
 			break;
 
+		 case OPT_list_supported_images:
+			list_supported_images();
+			exit(0);
+
 		 case OPT_mathalpha:
 			opt_mathalpha = true;
 			break;
@@ -498,9 +505,11 @@ main(int ac, char *av[])
 				errx(1, "server must be specified");
 			}
 
+			string *token_str = NULL;
 			const char *token = NULL;
 			if (token_file) {
-				token = get_token(token_file);
+				token_str = get_token(token_file);
+				token = string_get(token_str);
 			} else if (is_home) {
 				errx(1, "Home timeline requires your access token");
 			}
@@ -526,6 +535,16 @@ version(void)
 		progname, progver, SAYAKA_RELDATE);
 }
 
+static void
+list_supported_images(void)
+{
+	char **names = image_get_loaderinfo();
+	for (uint i = 0; names[i] != NULL; i += 2) {
+		printf("%-10s %s\n", names[i], names[i + 1]);
+	}
+	free(names);
+}
+
 static void
 usage(void)
 {
@@ -574,6 +593,7 @@ help_all(void)
 "  --force-blurhash       : Show blurhash image instead of actual image\n"
 "  --help-all             : This help\n"
 "  --ipv4 / --ipv6        : Connect only IPv4/v6 for both stream and images\n"
+"  --list-supported-images: Show supported filetype and decoder list\n"
 "  --mathalpha            : Use alternate character for some MathAlpha chars\n"
 "  --misskey              : Set misskey mode (No other choices at this point)\n"
 "  --max-image-cols=<n>   : Set max number of images per line\n"
@@ -785,7 +805,7 @@ init_screen(void)
 static void
 invalidate_cache(void)
 {
-	char cmd[1024];
+	char cmd[PATH_MAX + 80];
 
 	// アイコンは1週間分くらいか。
 	snprintf(cmd, sizeof(cmd),
@@ -802,36 +822,30 @@ invalidate_cache(void)
 	}
 }
 
-// filename からトークンを取得して strdup したものを返す。
+// filename からトークンを取得して返す。
 // 失敗するとその場でエラー終了する。
-static const char *
+static string *
 get_token(const char *filename)
 {
-	char buf[64];
 	FILE *fp;
-	const char *token;
+	string *token;
 
 	assert(filename);
 
-	memset(buf, 0, sizeof(buf));
 	fp = fopen(filename, "r");
 	if (fp == NULL) {
 		err(1, "%s", filename);
 	}
-	// 1行目だけ。
-	if (fgets(buf, sizeof(buf), fp)) {
-		chomp(buf);
-	}
+	token = string_fgets(fp);
 	fclose(fp);
-
-	if (buf[0] == '\0') {
+	if (token == NULL) {
+		errx(1, "%s: No token found", filename);
+	}
+	string_rtrim_inplace(token);
+	if (string_len(token) == 0) {
 		errx(1, "%s: No token found", filename);
 	}
 
-	token = strdup(buf);
-	if (token == NULL) {
-		err(1, "%s: strdup", __func__);
-	}
 	return token;
 }
 
