@@ -1,7 +1,7 @@
 /* vi:set ts=4: */
 /*
  * Copyright (C) 2015 Y.Sugahara (moveccr)
- * Copyright (C) 2021-2025 Tetsuya Isaki
+ * Copyright (C) 2021-2026 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -40,17 +40,18 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 
-#define SIXELV_VERSION "3.8.5"
-#define SIXELV_RELDATE "2025/12/30"
+#define SIXELV_VERSION "3.8.6"
+#define SIXELV_RELDATE "2026/01/28"
 
 typedef enum {
 	OUTPUT_FORMAT_SIXEL,
 	OUTPUT_FORMAT_BMP,
 	OUTPUT_FORMAT_ASCII,
+	OUTPUT_FORMAT_NULL,
 } OutputFormat;
 
 static void version(void);
-static void list(void);
+static void list_supported_images(void);
 static void usage(void);
 static void help_all(void);
 static bool do_file(const char *filename);
@@ -65,6 +66,7 @@ static uint fontheight;				// フォント高さ (ドット数)
 static bool show_filename;			// 画像の前にファイル名を表示
 static bool ignore_error;			// true ならエラーでも次ファイルを処理
 static FILE *ofp;					// 出力中のストリーム
+static bool opt_blurhash;			// 入力を Blurhash とみなす
 static bool opt_blurhash_nearest;	// Blurhash を最近傍補間する
 static ResizeAxis opt_resize_axis;
 static bool opt_no_progressive;
@@ -87,6 +89,7 @@ const char progver[]  = SIXELV_VERSION;
 
 enum {
 	OPT__start = 0x7f,
+	OPT_blurhash,
 	OPT_blurhash_nearest,
 	OPT_cdm,
 	OPT_ciphers,
@@ -97,9 +100,10 @@ enum {
 	OPT_height,
 	OPT_help,
 	OPT_help_all,
+	OPT_ignore_error,
 	OPT_ipv4,
 	OPT_ipv6,
-	OPT_list,
+	OPT_list_supported_images,
 	OPT_no_progressive,
 	OPT_output_format,
 	OPT_profile,
@@ -112,6 +116,7 @@ enum {
 };
 
 static const struct option longopts[] = {
+	{ "blurhash",		no_argument,		NULL,	'b' },
 	{ "blurhash-nearest",no_argument,		NULL,	OPT_blurhash_nearest },
 	{ "bn",				no_argument,		NULL,	OPT_blurhash_nearest },
 	{ "ciphers",		required_argument,	NULL,	OPT_ciphers },
@@ -125,10 +130,10 @@ static const struct option longopts[] = {
 	{ "height",			required_argument,	NULL,	'h' },
 	{ "help",			no_argument,		NULL,	OPT_help, },
 	{ "help-all",		no_argument,		NULL,	OPT_help_all },
-	{ "ignore-error",	no_argument,		NULL,	'i' },
+	{ "ignore-error",	no_argument,		NULL,	OPT_ignore_error },
 	{ "ipv4",			no_argument,		NULL,	OPT_ipv4 },
 	{ "ipv6",			no_argument,		NULL,	OPT_ipv6 },
-	{ "list",			no_argument,		NULL,	OPT_list },
+	{ "list-supported-images", no_argument,	NULL,	OPT_list_supported_images },
 	{ "no-progressive",	no_argument,		NULL,	OPT_no_progressive },
 	{ "output-format",	required_argument,	NULL,	'O' },
 	{ "page",			required_argument,	NULL,	'p' },
@@ -146,6 +151,7 @@ static const struct option longopts[] = {
 static const struct optmap map_output_format[] = {
 	{ "ascii",		OUTPUT_FORMAT_ASCII },
 	{ "bmp",		OUTPUT_FORMAT_BMP },
+	{ "null",		OUTPUT_FORMAT_NULL },
 	{ "sixel",		OUTPUT_FORMAT_SIXEL },
 	{ NULL },
 };
@@ -217,10 +223,14 @@ main(int ac, char *av[])
 	output_filename = NULL;
 	output_format = OUTPUT_FORMAT_SIXEL;
 
-	while ((c = getopt_long(ac, av, "c:d:h:iO:o:p:r:vw:",
+	while ((c = getopt_long(ac, av, "bc:d:h:iO:o:p:r:vw:",
 					longopts, NULL)) != -1)
 	{
 		switch (c) {
+		 case 'b':
+			opt_blurhash = true;
+			break;
+
 		 case OPT_blurhash_nearest:
 			opt_blurhash_nearest = true;
 			break;
@@ -301,6 +311,11 @@ main(int ac, char *av[])
 			exit(0);
 
 		 case 'i':
+			// --debug-image=1 の短縮形。
+			diag_set_level(diag_image, 1);
+			break;
+
+		 case OPT_ignore_error:
 			ignore_error = true;
 			break;
 
@@ -312,8 +327,8 @@ main(int ac, char *av[])
 			netopt.address_family = 6;
 			break;
 
-		 case OPT_list:
-			list();
+		 case OPT_list_supported_images:
+			list_supported_images();
 			exit(0);
 
 		 case 'O':
@@ -464,11 +479,11 @@ version(void)
 }
 
 static void
-list(void)
+list_supported_images(void)
 {
 	char **names = image_get_loaderinfo();
 	for (uint i = 0; names[i] != NULL; i += 2) {
-		printf("%-8s %s\n", names[i], names[i + 1]);
+		printf("%-10s %s\n", names[i], names[i + 1]);
 	}
 	free(names);
 }
@@ -479,34 +494,14 @@ usage(void)
 	printf("usage: %s [<options...>] [-|<file|url...>]\n", progname);
 	printf(
 "  -c <color>      : Color mode. 2, 8, 16, 256, gray[2..256] (default:256)\n"
-"                    (See --help-all for more details)\n"
 "  -w <width>      : Resize width to <width> pixel\n"
 "  -h <height>     : Resize height to <height> pixel\n"
 "  -r <method>     : Reduction method, none(simple) or high (default:high)\n"
-"  -O <fmt>        : Output format, ascii, bmp or sixel (default: sixel)\n"
+"  -O <fmt>        : Output format, ascii, bmp, null or sixel (default:sixel)\n"
 "  -o <filename>   : Output filename, '-' means stdout (default: -)\n"
-"  -p <page>       : Specify the page(frame). (animated GIF/WebP only)\n"
-"  -v              : Show input filename\n");
-
-	static const char * const opts[] = {
-		"-d <diffusion>",
-		"--resize-axis=<axis>",
-		"--gain=<gain>",
-		"--blurhash-nearest",
-		"--sixel-or",
-		"--sixel-transbg",
-		"--suppress-palette",
-		"--ignore-error",
-		"--ciphers=<ciphers>",
-		"--help-all",
-		"--debug-image=<0..2>",
-		"--debug-net=<0..2>",
-		"--debug-sixel=<0..2>",
-		"", // 偶数パディング
-	};
-	for (uint i = 0; i < countof(opts) / 2; i++) {
-		printf("  %-39s%s\n", opts[i * 2], opts[i * 2 + 1]);
-	}
+"  -p <page>       : Specify the page(frame). (GIF/ICO/WebP)\n"
+"  -v              : Show input filename\n"
+" See --help-all for all options and more details\n");
 }
 
 static void
@@ -542,27 +537,32 @@ help_all(void)
 "     2        : 2-pixels (right, down)\n"
 "     3        : 3-pixels (right, down, rightdown)\n"
 "     none     : No diffution\n"
+"\n" // ここからアルファベット順
+"  -b,--blurhash          : Input as Blurhash\n"
 "  --bn,--blurhash-nearest\n"
 "  --cdm=<value>          : Differential Color Diffusion Attenuator,\n"
 "                           between 0.0 and 1.0 (default:1.0)\n"
-"  --gain=<gain>          : Set output gain between 0.0 and 2.0 (default:1.0)\n"
-"  --help-all             : This help\n"
-"  --list                 : Show supported filetype and decoder list\n"
-"  -O,--output-format=<fmt> : ascii, bmp or sixel (default:sixel)\n"
-"  -o <filename>          : Output filename, '-' means stdout (default:-)\n"
-"  -p,--page=<page>       : Specify the page(frame). (animated GIF/WebP only)\n"
-"  --no-progressive       : Don't use progressive data (jxl only)\n"
-"  --sixel-or             : Output SIXEL by OR-mode\n"
-"  --sixel-transbg        : Make SIXEL background transparent\n"
-"  --suppress-palette     : Suppress output of SIXEL palette definition\n"
 "  --ciphers <ciphers>    : \"RSA\" can only be specified\n"
-"  --ipv4 / --ipv6        : Connect only IPv4/v6\n"
-"  -i,--ignore-error\n"
-"  -v                     : Show input filename\n"
-"  --version\n"
 "  --debug-image=<0..2>\n"
 "  --debug-net  =<0..2>\n"
 "  --debug-sixel=<0..2>\n"
+"  --gain=<gain>          : Set output gain between 0.0 and 2.0 (default:1.0)\n"
+"  --help                 : Short help\n"
+"  --help-all             : This help\n"
+"  -i                     : Synonym for --debug-image=1\n"
+"  --ipv4 / --ipv6        : Connect only IPv4/v6\n"
+"  --ignore-error\n"
+"  --list-supported-images: Show supported filetype and decoder list\n"
+"  --no-progressive       : Don't use progressive data (jxl only)\n"
+"  -O,--output-format=<fmt> : ascii, bmp, null or sixel (default:sixel)\n"
+"  -o <filename>          : Output filename, '-' means stdout (default:-)\n"
+"  -p,--page=<page>       : Specify the page(frame). (GIF/ICO/WebP)\n"
+"  --profile\n"
+"  --sixel-or             : Output SIXEL by OR-mode\n"
+"  --sixel-transbg        : Make SIXEL background transparent\n"
+"  --suppress-palette     : Suppress output of SIXEL palette definition\n"
+"  -v                     : Show input filename\n"
+"  --version\n"
 	);
 	exit(0);
 }
@@ -649,29 +649,35 @@ do_file(const char *infile)
 	PROF(&load_start);
 
 	// 画像形式判定。
-	int loader_idx = image_match(pstream, diag_image);
-
-	if (loader_idx >= 0) {
-		// 読み込み。
-		// この hint は libjpeg の scaling hint のことで、これをもとに
-		// 1/8 とかで読み込んだものが srcimg->{width,height} になる。
-		// ASCII の時には hint サイズも加工したほうが効率はいいがとりあえず。
-		memset(&hint, 0, sizeof(hint));
-		hint.axis   = opt_resize_axis;
-		hint.width  = opt_width;
-		hint.height = opt_height;
-		hint.page   = opt_page;
-		hint.progressive = !opt_no_progressive;
-		srcimg = image_read(pstream, loader_idx, &hint, diag_image);
-		if (srcimg) {
-			// 得られた画像サイズと引数指定から、いい感じにサイズを決定。
-			image_get_preferred_size(srcimg->width, srcimg->height,
-				opt_resize_axis, opt_width, opt_height,
-				&dst_width, &dst_height);
-		}
-	} else {
-		// どの画像形式でもなさそうなら Blurhash を試す。
+	if (opt_blurhash) {
+		// Blurhash はマジックも何もないで指定されないと分からないのと、
+		// このデータが Blurhash かどうかの判定も出来ないため、
+		// 他の知ってる画像形式にマッチしなかったからといって Blurhash に
+		// フォールバックしても何か無意味な画像が表示出来てしまうだけ
+		// 余計に紛らわしいため、必ずオプションで指定する。
 		srcimg = read_blurhash(pstream, &dst_width, &dst_height);
+	} else {
+		int loader_idx = image_match(pstream, diag_image);
+		if (loader_idx >= 0) {
+			// 読み込み。
+			// この hint は libjpeg の scaling hint のことで、これをもとに
+			// 1/8 とかで読み込んだものが srcimg->{width,height} になる。
+			// ASCII の時には hint サイズも加工したほうが効率はいいが
+			// とりあえず。
+			memset(&hint, 0, sizeof(hint));
+			hint.axis   = opt_resize_axis;
+			hint.width  = opt_width;
+			hint.height = opt_height;
+			hint.page   = opt_page;
+			hint.no_progressive = opt_no_progressive;
+			srcimg = image_read(pstream, loader_idx, &hint, diag_image);
+			if (srcimg) {
+				// 得られた画像サイズと引数指定から、いい感じにサイズを決定。
+				image_get_preferred_size(srcimg->width, srcimg->height,
+					opt_resize_axis, opt_width, opt_height,
+					&dst_width, &dst_height);
+			}
+		}
 	}
 
 	PROF(&load_end);
@@ -719,38 +725,43 @@ do_file(const char *infile)
 	}
 
 	// 出力先をオープン。
-	if (output_filename == NULL) {
-		ofp = stdout;
-	} else {
-		ofp = fopen(output_filename, "w");
-		if (ofp == NULL) {
-			warn("fopen(%s) failed", output_filename);
-			goto abort;
+	if (__predict_true(output_format != OUTPUT_FORMAT_NULL)) {
+		if (output_filename == NULL) {
+			ofp = stdout;
+		} else {
+			ofp = fopen(output_filename, "w");
+			if (ofp == NULL) {
+				warn("fopen(%s) failed", output_filename);
+				goto abort;
+			}
 		}
+
+		PROF(&sixel_start);
+
+		// 書き出し。
+		switch (output_format) {
+		 case OUTPUT_FORMAT_SIXEL:
+			image_sixel_write(ofp, resimg, &imageopt, diag_sixel);
+			break;
+		 case OUTPUT_FORMAT_BMP:
+			if (image_bmp_write(ofp, resimg, diag_image) == false) {
+				goto abort;
+			}
+			break;
+		 case OUTPUT_FORMAT_ASCII:
+			if (image_ascii_write(ofp, resimg, &imageopt, diag_image) == false){
+				goto abort;
+			}
+			break;
+		 case OUTPUT_FORMAT_NULL:
+			__unreachable();
+			break;
+		}
+		fflush(ofp);
+
+		PROF(&sixel_end);
 	}
 
-	PROF(&sixel_start);
-
-	// 書き出し。
-	switch (output_format) {
-	 case OUTPUT_FORMAT_SIXEL:
-		image_sixel_write(ofp, resimg, &imageopt, diag_sixel);
-		break;
-	 case OUTPUT_FORMAT_BMP:
-		if (image_bmp_write(ofp, resimg, diag_image) == false) {
-			goto abort;
-		}
-		break;
-	 case OUTPUT_FORMAT_ASCII:
-		if (image_ascii_write(ofp, resimg, &imageopt, diag_image) == false) {
-			goto abort;
-		}
-		break;
-	}
-	fflush(ofp);
-
-	PROF(&sixel_end);
-
 	if (opt_profile) {
 		uint64 load_usec;
 		uint64 cvt_usec;
