@@ -1,6 +1,6 @@
 /* vi:set ts=4: */
 /*
- * Copyright (C) 2021-2024 Tetsuya Isaki
+ * Copyright (C) 2021-2025 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -108,46 +108,6 @@ test_base64_encode(void)
 	}
 }
 
-static void
-test_chomp(void)
-{
-	printf("%s\n", __func__);
-
-	struct {
-		const char *src;
-		const char *exp;
-	} table[] = {
-		{ "",			"" },
-		{ "abc",		"abc" },
-		{ " abc",		" abc" },
-		{ "  abc",		"  abc" },
-		{ "a ",			"a " },
-		{ "a  ",		"a  " },
-		{ "  ab  ",		"  ab  " },
-		{ "a\r\n",		"a" },
-		{ "\r\r\n\n",	"" },
-		{ "\n ab\t \n",	"\n ab\t " },
-	};
-	for (uint i = 0; i < countof(table); i++) {
-		char buf[64];
-		const char *src = table[i].src;
-		const char *exp = table[i].exp;
-
-		strlcpy(buf, src, sizeof(buf));
-		chomp(buf);
-		if (strcmp(exp, buf) != 0) {
-			string *src_esc = string_escape_c(src);
-			string *act_esc = string_escape_c(buf);
-			string *exp_esc = string_escape_c(exp);
-			fail("\"%s\" expects \"%s\" but \"%s\"\n",
-				string_get(src_esc), string_get(exp_esc), string_get(act_esc));
-			string_free(src_esc);
-			string_free(act_esc);
-			string_free(exp_esc);
-		}
-	}
-}
-
 static void
 test_decode_isotime(void)
 {
@@ -451,11 +411,15 @@ test_string_rtrim_inplace(void)
 		const char *exp;
 	} table[] = {
 		// input		expected
+		{ "",			"" },
 		{ "ab c",		"ab c" },
+		{ " abc",		" abc" },
+		{ "a ",			"a" },
+		{ "a  ",		"a" },
+		{ "  ab  ",		"  ab" },
 		{ "ab c \n",	"ab c" },
 		{ "a\t \r \n",	"a" },
 		{ "\r\n",		"" },
-		{ "",			"" },
 	};
 	for (uint i = 0; i < countof(table); i++) {
 		const char *src = table[i].src;
@@ -581,7 +545,6 @@ main(int ac, char *av[])
 	}
 
 	test_base64_encode();
-	test_chomp();
 	test_decode_isotime();
 	test_json_unescape();
 	test_putd();
