@@ -1,6 +1,6 @@
 /* vi:set ts=4: */
 /*
- * Copyright (C) 2016-2024 Tetsuya Isaki
+ * Copyright (C) 2016-2025 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -44,20 +44,6 @@ strerrno(void)
 	return strerror(errno);
 }
 
-// 文字列 s 末尾の連続する改行を取り除く。
-void
-chomp(char *s)
-{
-	char *p = s + strlen(s);
-	while (--p >= s) {
-		if (*p == '\r' || *p == '\n') {
-			*p = '\0';
-		} else {
-			break;
-		}
-	}
-}
-
 // map から arg に対応する値を返す。
 // 見付からなければ -1 を返す。
 int
